@@ -117,18 +117,23 @@ def conversion_transfer(conversion, version=3):
 
                 elif conversion_type == v4c.CONVERSION_TYPE_TABX:
                     nr = conversion.val_param_nr
+
                     kargs = {
-                        "ref_param_nr": nr,
+                        "ref_param_nr": nr + 1,
                         "unit": unit,
-                        "conversion_type": v3c.CONVERSION_TYPE_TABX,
+                        "conversion_type": v3c.CONVERSION_TYPE_RTABX,
                     }
                     for i in range(nr):
-                        kargs[f"param_val_{i}"] = conversion[f"val_{i}"]
-                        kargs[f"text_{i}"] = conversion.referenced_blocks[
-                            f"text_{i}"
-                        ].text
+                        kargs[f"lower_{i}"] = conversion[f"val_{i}"]
+                        kargs[f"upper_{i}"] = conversion[f"val_{i}"]
+                        kargs[f"text_{i}"] = conversion.referenced_blocks[f"text_{i}"]
 
-                    conversion = v3b.ChannelConversion(**kargs)
+                    new_conversion = v3b.ChannelConversion(**kargs)
+                    new_conversion.referenced_blocks[
+                        "default_addr"
+                    ] = conversion.referenced_blocks["default_addr"]
+
+                    conversion = new_conversion
 
                 elif conversion_type == v4c.CONVERSION_TYPE_RTABX:
                     nr = conversion.val_param_nr // 2
@@ -140,16 +145,12 @@ def conversion_transfer(conversion, version=3):
                     for i in range(nr):
                         kargs[f"lower_{i}"] = conversion[f"lower_{i}"]
                         kargs[f"upper_{i}"] = conversion[f"upper_{i}"]
-                        kargs[f"text_{i}"] = conversion.referenced_blocks[
-                            f"text_{i}"
-                        ].text
+                        kargs[f"text_{i}"] = conversion.referenced_blocks[f"text_{i}"]
 
                     new_conversion = v3b.ChannelConversion(**kargs)
-
-                    new_conversion.referenced_blocks["default_addr"] = v3b.TextBlock(
-                        text=conversion.referenced_blocks["default_addr"].text
-                    )
-                    new_conversion
+                    new_conversion.referenced_blocks[
+                        "default_addr"
+                    ] = conversion.referenced_blocks["default_addr"]
 
                     conversion = new_conversion
 
@@ -192,7 +193,7 @@ def conversion_transfer(conversion, version=3):
             elif conversion_type == v3c.CONVERSION_TYPE_TAB:
                 conversion_ = {}
                 conversion_["val_param_nr"] = conversion.ref_param_nr * 2
-                for i in range(conversion.val_param_nr):
+                for i in range(conversion.ref_param_nr):
                     conversion_[f"raw_{i}"] = conversion[f"raw_{i}"]
                     conversion_[f"phys_{i}"] = conversion[f"phys_{i}"]
 
@@ -232,12 +233,12 @@ def conversion_transfer(conversion, version=3):
                     "val_param_nr": nr * 2,
                     "ref_param_nr": nr + 1,
                     "conversion_type": v4c.CONVERSION_TYPE_RTABX,
-                    "default_addr": conversion.referenced_blocks["default_addr"].text,
+                    "default_addr": conversion.referenced_blocks["default_addr"],
                 }
                 for i in range(nr):
                     kargs[f"lower_{i}"] = conversion[f"lower_{i}"]
                     kargs[f"upper_{i}"] = conversion[f"upper_{i}"]
-                    kargs[f"text_{i}"] = conversion.referenced_blocks[f"text_{i}"].text
+                    kargs[f"text_{i}"] = conversion.referenced_blocks[f"text_{i}"]
 
                 conversion = v4b.ChannelConversion(**kargs)
 
@@ -295,8 +296,6 @@ def from_dict(conversion):
         conversion = v4b.ChannelConversion(**conversion)
 
     else:
-        conversion = v4b.ChannelConversion(
-            conversion_type=v4c.CONVERSION_TYPE_NON
-        )
+        conversion = v4b.ChannelConversion(conversion_type=v4c.CONVERSION_TYPE_NON)
 
     return conversion
