@@ -9,6 +9,7 @@ import xml.etree.ElementTree as ET
 import re
 import subprocess
 from io import BytesIO
+import sys
 
 from collections import namedtuple
 from random import randint
@@ -263,7 +264,7 @@ def sanitize_xml(text):
     return re.sub(_xmlns_pattern, "", text)
 
 
-def get_fmt_v3(data_type, size):
+def get_fmt_v3(data_type, size, byte_order=v3c.BYTE_ORDER_INTEL):
     """convert mdf versions 2 and 3 channel data type to numpy dtype format
     string
 
@@ -279,46 +280,62 @@ def get_fmt_v3(data_type, size):
         numpy compatible data type format string
 
     """
-    if data_type in {v3c.DATA_TYPE_STRING, v3c.DATA_TYPE_BYTEARRAY}:
+    if data_type in (v3c.DATA_TYPE_STRING, v3c.DATA_TYPE_BYTEARRAY):
         size = size // 8
         if data_type == v3c.DATA_TYPE_STRING:
             fmt = f"S{size}"
-        elif data_type == v3c.DATA_TYPE_BYTEARRAY:
+        else:
             fmt = f"({size},)u1"
     else:
-        if size <= 8:
-            size = 1
-        elif size <= 16:
-            size = 2
-        elif size <= 32:
-            size = 4
-        elif size <= 64:
-            size = 8
+        if size > 64 and data_type in (v3c.DATA_TYPE_UNSIGNED_INTEL, v3c.DATA_TYPE_UNSIGNED, v3c.DATA_TYPE_UNSIGNED_MOTOROLA):
+            fmt = f"({size // 8},)u1"
         else:
-            size = size // 8
+            if size <= 8:
+                size = 1
+            elif size <= 16:
+                size = 2
+            elif size <= 32:
+                size = 4
+            elif size <= 64:
+                size = 8
+            else:
+                size = size // 8
 
-        if data_type in (v3c.DATA_TYPE_UNSIGNED_INTEL, v3c.DATA_TYPE_UNSIGNED):
-            fmt = f"<u{size}".format()
+            if data_type == v3c.DATA_TYPE_UNSIGNED_INTEL:
+                fmt = f"<u{size}"
 
-        elif data_type == v3c.DATA_TYPE_UNSIGNED_MOTOROLA:
-            fmt = f">u{size}"
+            elif data_type == v3c.DATA_TYPE_UNSIGNED:
+                if byte_order == v3c.BYTE_ORDER_INTEL:
+                    fmt = f"<u{size}"
+                else:
+                    fmt = f">u{size}"
 
-        elif data_type in (v3c.DATA_TYPE_SIGNED_INTEL, v3c.DATA_TYPE_SIGNED):
-            fmt = f"<i{size}"
+            elif data_type == v3c.DATA_TYPE_UNSIGNED_MOTOROLA:
+                fmt = f">u{size}"
 
-        elif data_type == v3c.DATA_TYPE_SIGNED_MOTOROLA:
-            fmt = f">i{size}"
+            elif data_type == v3c.DATA_TYPE_SIGNED_INTEL:
+                fmt = f"<i{size}"
 
-        elif data_type in {
-            v3c.DATA_TYPE_FLOAT,
-            v3c.DATA_TYPE_DOUBLE,
-            v3c.DATA_TYPE_FLOAT_INTEL,
-            v3c.DATA_TYPE_DOUBLE_INTEL,
-        }:
-            fmt = f"<f{size}"
+            elif data_type == v3c.DATA_TYPE_SIGNED:
+                if byte_order == v3c.BYTE_ORDER_INTEL:
+                    fmt = f"<i{size}"
+                else:
+                    fmt = f">i{size}"
 
-        elif data_type in (v3c.DATA_TYPE_FLOAT_MOTOROLA, v3c.DATA_TYPE_DOUBLE_MOTOROLA):
-            fmt = f">f{size}"
+            elif data_type == v3c.DATA_TYPE_SIGNED_MOTOROLA:
+                fmt = f">i{size}"
+
+            elif data_type in (v3c.DATA_TYPE_FLOAT_INTEL, v3c.DATA_TYPE_DOUBLE_INTEL):
+                fmt = f"<f{size}"
+
+            elif data_type in (v3c.DATA_TYPE_FLOAT_MOTOROLA, v3c.DATA_TYPE_DOUBLE_MOTOROLA):
+                fmt = f">f{size}"
+
+            elif data_type in (v3c.DATA_TYPE_FLOAT, v3c.DATA_TYPE_DOUBLE):
+                if byte_order == v3c.BYTE_ORDER_INTEL:
+                    fmt = f"<f{size}"
+                else:
+                    fmt = f">f{size}"
 
     return fmt
 
@@ -369,35 +386,41 @@ def get_fmt_v4(data_type, size, channel_type=v4c.CHANNEL_TYPE_VALUE):
             fmt = "V6"
 
     else:
-
-        if size <= 8:
-            size = 1
-        elif size <= 16:
-            size = 2
-        elif size <= 32:
-            size = 4
-        elif size <= 64:
-            size = 8
+        if size > 64 and data_type in (v4c.DATA_TYPE_UNSIGNED_INTEL, v4c.DATA_TYPE_UNSIGNED):
+            fmt = f"({size // 8},)u1"
         else:
-            size = size // 8
+            if size <= 8:
+                size = 1
+            elif size <= 16:
+                size = 2
+            elif size <= 32:
+                size = 4
+            elif size <= 64:
+                size = 8
+            else:
+                size = size // 8
 
-        if data_type == v4c.DATA_TYPE_UNSIGNED_INTEL:
-            fmt = f"<u{size}"
+            if data_type == v4c.DATA_TYPE_UNSIGNED_INTEL:
+                fmt = f"<u{size}"
 
-        elif data_type == v4c.DATA_TYPE_UNSIGNED_MOTOROLA:
-            fmt = f">u{size}"
+            elif data_type == v4c.DATA_TYPE_UNSIGNED_MOTOROLA:
+                fmt = f">u{size}"
 
-        elif data_type == v4c.DATA_TYPE_SIGNED_INTEL:
-            fmt = f"<i{size}"
+            elif data_type == v4c.DATA_TYPE_SIGNED_INTEL:
+                fmt = f"<i{size}"
 
-        elif data_type == v4c.DATA_TYPE_SIGNED_MOTOROLA:
-            fmt = f">i{size}"
+            elif data_type == v4c.DATA_TYPE_SIGNED_MOTOROLA:
+                fmt = f">i{size}"
 
-        elif data_type == v4c.DATA_TYPE_REAL_INTEL:
-            fmt = f"<f{size}"
+            elif data_type == v4c.DATA_TYPE_REAL_INTEL:
+                fmt = f"<f{size}"
 
-        elif data_type == v4c.DATA_TYPE_REAL_MOTOROLA:
-            fmt = f">f{size}"
+            elif data_type == v4c.DATA_TYPE_REAL_MOTOROLA:
+                fmt = f">f{size}"
+            elif data_type == v4c.DATA_TYPE_COMPLEX_INTEL:
+                fmt = f"<c{size}"
+            elif data_type == v4c.DATA_TYPE_COMPLEX_MOTOROLA:
+                fmt = f">c{size}"
 
     return fmt
 
@@ -421,25 +444,29 @@ def fmt_to_datatype_v3(fmt, shape, array=False):
         integer data type as defined by ASAM MDF and bit size
 
     """
+    byteorder = fmt.byteorder
+    if byteorder == '=':
+        byteorder = '<' if sys.byteorder == 'little' else '>'
     size = fmt.itemsize * 8
+    kind = fmt.kind
 
-    if not array and shape[1:] and fmt.itemsize == 1 and fmt.kind == "u":
+    if not array and shape[1:] and fmt.itemsize == 1 and kind == "u":
         data_type = v3c.DATA_TYPE_BYTEARRAY
         for dim in shape[1:]:
             size *= dim
     else:
-        if fmt.kind == "u":
-            if fmt.byteorder in "=<|":
-                data_type = v3c.DATA_TYPE_UNSIGNED
+        if kind == "u":
+            if byteorder in "<|":
+                data_type = v3c.DATA_TYPE_UNSIGNED_INTEL
             else:
                 data_type = v3c.DATA_TYPE_UNSIGNED_MOTOROLA
-        elif fmt.kind == "i":
-            if fmt.byteorder in "=<|":
-                data_type = v3c.DATA_TYPE_SIGNED
+        elif kind == "i":
+            if byteorder in "<|":
+                data_type = v3c.DATA_TYPE_SIGNED_INTEL
             else:
                 data_type = v3c.DATA_TYPE_SIGNED_MOTOROLA
-        elif fmt.kind == "f":
-            if fmt.byteorder in "=<":
+        elif kind == "f":
+            if byteorder in "<":
                 if size == 32:
                     data_type = v3c.DATA_TYPE_FLOAT
                 else:
@@ -449,10 +476,10 @@ def fmt_to_datatype_v3(fmt, shape, array=False):
                     data_type = v3c.DATA_TYPE_FLOAT_MOTOROLA
                 else:
                     data_type = v3c.DATA_TYPE_DOUBLE_MOTOROLA
-        elif fmt.kind in "SV":
+        elif kind in "SV":
             data_type = v3c.DATA_TYPE_STRING
-        elif fmt.kind == "b":
-            data_type = v3c.DATA_TYPE_UNSIGNED
+        elif kind == "b":
+            data_type = v3c.DATA_TYPE_UNSIGNED_INTEL
             size = 1
         else:
             message = f"Unknown type: dtype={fmt}, shape={shape}"
@@ -512,28 +539,30 @@ def fmt_to_datatype_v4(fmt, shape, array=False):
         integer data type as defined by ASAM MDF and bit size
 
     """
+    byteorder = fmt.byteorder
+    if byteorder == '=':
+        byteorder = '<' if sys.byteorder == 'little' else '>'
     size = fmt.itemsize * 8
+    kind = fmt.kind
 
-    if not array and shape[1:] and fmt.itemsize == 1 and fmt.kind == "u":
+    if not array and shape[1:] and fmt.itemsize == 1 and kind == "u":
         data_type = v4c.DATA_TYPE_BYTEARRAY
         for dim in shape[1:]:
             size *= dim
 
     else:
-        kind = fmt.kind
-        byteorder = fmt.byteorder
         if kind == "u":
-            if byteorder in "=<|":
+            if byteorder in "<|":
                 data_type = v4c.DATA_TYPE_UNSIGNED_INTEL
             else:
                 data_type = v4c.DATA_TYPE_UNSIGNED_MOTOROLA
         elif kind == "i":
-            if byteorder in "=<|":
+            if byteorder in "<|":
                 data_type = v4c.DATA_TYPE_SIGNED_INTEL
             else:
                 data_type = v4c.DATA_TYPE_SIGNED_MOTOROLA
         elif kind == "f":
-            if byteorder in "=<":
+            if byteorder in "<":
                 data_type = v4c.DATA_TYPE_REAL_INTEL
             else:
                 data_type = v4c.DATA_TYPE_REAL_MOTOROLA
@@ -542,6 +571,11 @@ def fmt_to_datatype_v4(fmt, shape, array=False):
         elif kind == "b":
             data_type = v4c.DATA_TYPE_UNSIGNED_INTEL
             size = 1
+        elif kind == "c":
+            if byteorder in "<":
+                data_type = v4c.DATA_TYPE_COMPLEX_INTEL
+            else:
+                data_type = v4c.DATA_TYPE_COMPLEX_MOTOROLA
         else:
             message = f"Unknown type: dtype={fmt}, shape={shape}"
             logger.exception(message)
@@ -1340,18 +1374,52 @@ def extract_can_signal(signal, payload):
     big_endian = False if signal.is_little_endian else True
     signed = signal.is_signed
 
-    start_bit = signal.get_startbit()
+    start_bit = signal.get_startbit(bit_numbering=1)
 
-    start_byte, bit_offset = divmod(start_bit, 8)
+    if big_endian:
+        start_byte = start_bit // 8
+        bit_count = signal.size
+
+        pos = start_bit % 8 + 1
+
+        over = bit_count % 8
+
+        if pos >= over:
+            bit_offset = (pos - over) % 8
+        else:
+            bit_offset = pos + 8 - over
+    else:
+        start_byte, bit_offset = divmod(start_bit, 8)
 
     bit_count = signal.size
 
-    if start_bit + bit_count > vals.shape[1] * 8:
-        raise MdfException(
-            f'Could not extract signal "{signal.name}" with start '
-            f"bit {signal.get_startbit()} and bit count {signal.size} "
-            f"from the payload with shape {vals.shape}"
-        )
+    if big_endian:
+        byte_pos = start_byte + 1
+        start_pos = start_bit
+        bits = bit_count
+
+        while True:
+            pos = start_pos % 8 + 1
+            if pos < bits:
+                byte_pos += 1
+                bits -= pos
+                start_pos = 7
+            else:
+                break
+
+        if byte_pos > vals.shape[1] * 8:
+            raise MdfException(
+                f'Could not extract signal "{signal.name}" with start '
+                f"bit {start_bit} and bit count {signal.size} "
+                f"from the payload with shape {vals.shape}"
+            )
+    else:
+        if start_bit + bit_count > vals.shape[1] * 8:
+            raise MdfException(
+                f'Could not extract signal "{signal.name}" with start '
+                f"bit {start_bit} and bit count {signal.size} "
+                f"from the payload with shape {vals.shape}"
+            )
 
     byte_size, r = divmod(bit_offset + bit_count, 8)
     if r:
@@ -1367,20 +1435,26 @@ def extract_can_signal(signal, payload):
     # prepend or append extra bytes columns
     # to get a standard size number of bytes
 
+    #print(signal.name, start_bit, bit_offset, start_byte, byte_size)
+
     if extra_bytes:
         if big_endian:
 
             vals = np.column_stack(
                 [
-                    np.zeros(len(vals), dtype=f"<({extra_bytes},)u1"),
                     vals[:, start_byte : start_byte + byte_size],
+                    np.zeros(len(vals), dtype=f"<({extra_bytes},)u1"),
                 ]
             )
+
             try:
                 vals = vals.view(f">u{std_size}").ravel()
             except:
                 vals = np.frombuffer(vals.tobytes(), dtype=f">u{std_size}")
 
+            vals = vals >> (extra_bytes * 8 + bit_offset)
+            vals &= (2 ** bit_count) - 1
+
         else:
             vals = np.column_stack(
                 [
@@ -1393,6 +1467,9 @@ def extract_can_signal(signal, payload):
             except:
                 vals = np.frombuffer(vals.tobytes(), dtype=f"<u{std_size}")
 
+            vals = vals >> bit_offset
+            vals &= (2 ** bit_count) - 1
+
     else:
         if big_endian:
             try:
@@ -1406,6 +1483,9 @@ def extract_can_signal(signal, payload):
                     vals[:, start_byte : start_byte + byte_size].tobytes(),
                     dtype=f">u{std_size}",
                 )
+
+            vals = vals >> bit_offset
+            vals &= (2 ** bit_count) - 1
         else:
             try:
                 vals = (
@@ -1419,8 +1499,8 @@ def extract_can_signal(signal, payload):
                     dtype=f"<u{std_size}",
                 )
 
-    vals = vals >> bit_offset
-    vals &= (2 ** bit_count) - 1
+            vals = vals >> bit_offset
+            vals &= (2 ** bit_count) - 1
 
     if signed:
         vals = as_non_byte_sized_signed_int(vals, bit_count)
